:root{color-scheme:dark;--main: hsl(0deg 90% 48%);--main-2: hsl(0deg 90% 62%);--text-color: hsl(0deg 0% 90%);--secondary-text-color: hsl(0deg 0% 70%);--bg: fixed radial-gradient(ellipse at bottom, #1b2735 0%, #090a0f 100%);--secondary-bg: hsl(0deg 0% 9%);--tertiary-bg: hsl(0deg 0% 18%);--contrast: hsl(240deg 100% 50%);--contrast-2: hsl(195deg 100% 50%);--tp-bg-color: hsl(0deg 0% 0%)}#snow p{position:fixed;width:10px;height:10px;border-radius:50%;background:linear-gradient(#fff,#fff);filter:drop-shadow(0 0 10px white);font-size:0}main{text-shadow:0 0 10px white}header,#site-pane{box-shadow:0 0 10px #fff}textarea,select,input[type=email],input[type=range],input[type=search],input[type=text],input[type=number]{background:#000!important}
/*# sourceMappingURL=christmas.css.map */
