@import"/static/css/comic.css?v=0";:root{--main: hsl(48deg 100% 50%);--main-2: hsl(30deg 100% 50%);--text-color: hsl(106deg 100% 50%);--secondary-text-color: hsl(159deg 100% 50%);--bg: hsl(303deg 100% 50%);--secondary-bg: hsl(289deg 100% 50%);--tertiary-bg: hsl(264deg 100% 50%);--contrast: hsl(207deg 100% 50%);--contrast-2: hsl(195deg 100% 50%)}
/*# sourceMappingURL=fun.css.map */
