// @license magnet:?xt=urn:btih:d3d9a9a6595521f9666a5e94cc830dab83b65699&dn=expat.txt MIT
const s=document.getElementById("snow");let h=200;export function showSnow(n){n?s.style.display="block":s.style.display="none"}function p(n){const t=document.createElement("style");t.appendChild(document.createTextNode(n)),document.getElementsByTagName("head")[0].appendChild(t)}function b(n=100){return Math.floor(Math.random()*n)+1}function w(n,t){return n=Math.ceil(n),t=Math.floor(t),Math.floor(Math.random()*(t-n+1))+n}function y(n,t){return Math.random()*(t-n)+n}function g(n=200){let t="";for(let e=1;e<=n;e++){const o=Math.random()*100,r=Math.random()*10,d=o+r,i=o+r/2,l=y(.3,.8),c=l*100,a=Math.random(),m=w(10,30),u=b(30)*-1,f=Math.random();t+="#snow p:nth-child(".concat(e,"){opacity:").concat(f,";transform:translate(").concat(o,"vw,-10px) scale(").concat(a,");animation:fall-").concat(e," ").concat(m,"s ").concat(u,"s linear infinite}@keyframes fall-").concat(e,"{").concat(l*100,"%{transform:translate(").concat(d,"vw,").concat(c,"vh) scale(").concat(a,")}to{transform:translate(").concat(i,"vw,100vh) scale(").concat(a,")}}")}p(t)}export function createSnow(){g(h)}// @license-end
//# sourceMappingURL=snow.js.map
