{% extends "info.html" %}

{% block info_column %}
    Zitat: »{{quote}}«
    <br>
    Originaler Autor: <a href="{{fix_url(f'/zitate/info/a/{quote.author.id}')}}"
                        >{{quote.author}}</a>
{% end %}
