:root{color-scheme:dark;--main: hsl(0deg 90% 48%);--main-2: hsl(0deg 90% 62%);--text-color: hsl(0deg 0% 90%);--secondary-text-color: hsl(0deg 0% 70%);--bg: hsl(0deg 0% 0%);--secondary-bg: hsl(0deg 0% 9%);--tertiary-bg: hsl(0deg 0% 18%);--contrast: hsl(240deg 100% 50%);--contrast-2: hsl(195deg 100% 50%)}
/*# sourceMappingURL=default.css.map */
