:root{color-scheme:dark;--main: hsl(100deg 100% 40%);--main-2: hsl(100deg 100% 30%);--text-color: hsl(100deg 0% 90%);--secondary-text-color: hsl(250deg 0% 60%);--bg: hsl(0deg 0% 0%);--secondary-bg: hsl(0deg 0% 9%);--tertiary-bg: hsl(0deg 0% 18%);--contrast: hsl(0deg 90% 48%);--contrast-2: hsl(0deg 90% 62%)}
/*# sourceMappingURL=green.css.map */
