<footer id="footer" data-elastic-exclude>Mit Liebe gebacken
    {% if globals()["🦘"] %}
        {% raw emoji2html("🦘") %}
    {% elif now.microsecond % 4 == 3 and now.weekday() == 2 %}
        <span tooltip="ribbit">{% raw emoji2html("🐸") %}</span>
    {% elif globals()["🥚"] %}
        <a href="{{fix_url('https://bibliogram.froth.zone/p/BsOGulcndj-')}}"
           class="no-style-a-emoji">{% raw emoji2html("🥚") %}</a>
    {% elif now.month == now.day %}
        {% raw emoji2html(("🥇", "🥈", "🔱", "🍀", "🖐️", "🔯", "🎰", "✴️", "9⃣", "🔟", "🧝", "🕛️")[now.month - 1]) %}
    {% elif now.month == 2 and now.day == 29 %}
        {% raw emoji2html("🫀") %}
    {% elif now.month == 3 and now.day == 8 %}
        {% raw emoji2html("♀️") %}
    {% elif now.month == 3 and now.day == 14 %}
        {% raw emoji2html("🥧") %}
    {% elif now.month == 3 and now.day == 17 %}
        {% raw emoji2html("☘️") %}
    {% elif now.month == 3 and now.day == 25 %}
        {% raw emoji2html("🧇") %}
    {% elif now.month == 4 and now.day == 20 %}
        <a href="{{fix_url('https://y.com.sb/watch?v=C_mndbMCzU8')}}"
           class="no-style-a-emoji">{% raw emoji2html("🥦") %}</a>
    {% elif now.month == 4 and now.day == 21 and now.year == 2026 %}
        {% raw emoji2html("👑") %}
        {% raw emoji2html("🎂") %}
        {% raw emoji2html("💯") %}
    {% elif now.month == 4 and now.day == 22 %}
        <a href="{{fix_url('https://earthday.org')}}"
           class="no-style-a-emoji">{% raw emoji2html("🌍") %}</a>
    {% elif now.month == 4 and now.day == 23 %}
        <a href="{{fix_url('https://welttag-des-buches.de')}}"
           class="no-style-a-emoji"
        >{% raw emoji2html("📖") %}</a>
    {% elif now.month == 4 and now.day == 27 %}
        <a href="{{fix_url(f'https://stanleyparable{2 if now.utctimetuple().tm_hour >= 16 else str()}.com')}}"
           class="no-style-a-emoji">{% raw emoji2html("🪣") %}</a>
    {% elif now.month == 5 and now.day == 4 %}
        {% raw emoji2html("✨") %}
    {% elif now.month == 5 and now.day == 20 %}
        {% raw emoji2html("🐝") %}
    {% elif now.month == 6 and now.day == 24 %}
        {% raw emoji2html("🛸") %}
    {% elif now.month == 6 and now.day == 28 %}
        <a href="{{fix_url('https://tauday.com/tau-manifesto')}}"
           class="no-style-a-emoji">τ</a>
    {% elif now.month == 6 %}
        {% raw emoji2html(("❤", "🧡", "💛", "💚", "💙", "💜")[now.microsecond % 6]) %}
    {% elif now.month == 7 and now.day == 4 %}
        {% raw emoji2html("🗽") %}
    {% elif now.month == 7 and now.day == 17 %}
        {% from emoji import EMOJI_DATA %}
        {% set emoji = tuple(EMOJI_DATA)[now.microsecond % len(EMOJI_DATA)] %}
        <a href="{{fix_url(f'http://📙.la/{emoji}')}}"
           tooltip="{{EMOJI_DATA[emoji].get(lang, EMOJI_DATA[emoji]['en'])}}"
           class="no-style-a-emoji">{% raw emoji2html(emoji) %}</a>
    {% elif now.month == 9 and now.day == 11 %}
        {% raw emoji2html("✈️") %}
    {% elif now.month == 9 and now.day == 15 %}
        <a href="{{fix_url('https://sabrina-online.com')}}"
           class="no-style-a-emoji">{% raw emoji2html("🦨") %}</a>
    {% elif now.month == 9 and now.day == 19 %}
        {% raw emoji2html("☠️") %}
    {% elif now.month == 10 and now.day == 31 %}
        {% raw emoji2html("🎃") %}
    {% elif now.month == 12 and now.day in {24, 25} %}
        {% raw emoji2html("🎄") %}
    {% elif now.day == 13 and now.weekday() == 4 %}
        {% raw emoji2html("🐈‍⬛") %}
    {% elif not bumpscosity and now.microsecond % 2 %}
        <span tooltip="Achtung, es ist keine Bumpscosity vorhanden!">{% raw emoji2html("⚠️") %}</span>
    {% elif bumpscosity == 1000 and now.microsecond % 2  %}
        <span tooltip="Achtung, die Bumpscosity ist erschreckend hoch!">{% raw emoji2html("⚠️") %}</span>
    {% elif bumpscosity == 50 and now.microsecond % 2  %}
        <span tooltip="Die Bumpscosity ist echt angenehm.">{% raw emoji2html("🤍") %}</span>
    {% else %}
        {% raw emoji2html("🖤") %}
    {% end %}
    <a href="{{fix_url(GH_REPO_URL)}}">Asoziales Netzwerk: Sektion GitHub</a>
    {% if openmoji %}
        <br>
        <br>
        <div id="openmoji-attribution"
             openmoji-version="{{ openmoji_version }}"
             type="{{openmoji}}">  {# openmoji is truthy!! #}
            All emojis {% raw emoji2html("🥳") %} designed by
            <a href="{{fix_url('https://openmoji.org')}}">OpenMoji</a>
            – the open-source emoji and icon project.
            License: <a href="{{fix_url('https://creativecommons.org/licenses/by-sa/4.0/')}}">CC BY-SA 4.0</a>
        </div>
    {% end %}
</footer>
