table{margin-right:auto;margin-left:auto}th{width:45%}.rating{display:inline;color:var(--secondary-text-color)}.positive{color:green}.negative{color:red}.wrong-quote-a{color:var(--text-color)!important}#disclaimer{display:flex;flex-direction:row;margin-top:2rem}.button-container-a{display:inline-block;width:1rem;height:1rem}#report{display:block;width:3.2rem;height:3.2rem}.button-container-a img{width:100%;height:100%}#author-info{padding:.3125rem;border-right:.3125rem;background-color:var(--secondary-bg)}
/*# sourceMappingURL=info.css.map */
